﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using ZoiperAPI;

using System.Threading;
using System.Windows.Threading;

namespace zoiper_sdk
{


    /// <summary>
    /// This is an example of showing how to use Zoiper API
    /// with some of its methods
    /// </summary>
    public partial class MainWindow : Window
    {
        public bool bHold = true;            ///< A temp variable holds if a call is holded
        public string lastCall = "";         ///< A variable that holds the call time span
        public int numCalls = 0;             ///< A variable that holds the current call
        public IZoiperCall callTemp;         ///< A tempoary call variable for increace the call list
        public DispatcherTimer newTimer;     ///< A timer for showing the call duration
        public ZoiperAPI.ZoiperAPI myZoiper;      ///< A phone variable
        public List<IZoiperCall> calls = new List<IZoiperCall>();   ///< A list of calls
        public List<bool> bTimes = new List<bool>();                ///< A list of flags for every call
        public IZoiperAccount tempAccount;                          ///< A tempoary variable that holds an Accound


        /// <summary>
        /// This method appears when we or 
        /// someone on the other side ends the call
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallHang(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = "Down";
                btnHold.IsEnabled = false;
                txtLog.Text += "Last call: " + lastCall + "\n";
            });
            bTimes[numCalls] = false;
        }
        /// <summary>
        /// This method appears when the phone is ringing
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallRing(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = "Ringing: \n";
            });
            
        }
        /// <summary>
        /// This method appears when someone on the other side hold the call
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallHold(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = ("Remote Hold\n");
                btnHold.Content = "Unhold";
            });
        }
        /// <summary>
        /// This method appears when someone on the other side unhold the call
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallUnhold(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = ("Up\n");
                btnHold.Content = "Hold";
            });
        }
        /// <summary>
        /// This method appears when the call begins
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallAccept(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                bTimes[numCalls] = true;
                txtLable.Text = ("Up\n");
                newTimer.Start();
                txtLog.Text += iz.Account.Name + "\n";
                txtLog.Text += iz.Account.Type + "\n";
                //txtLog.Text += iz.Account.Status + "\n";
                //txtLog.Text += myZoiper.DefaultFilterText + "\n";
                //iz.Account.CreateAccount="1201"; //MuteMicrophone
            });
        }
        /// <summary>
        /// This method appears when someone calls us,
        /// to reject you have to use "iz" variable
        /// </summary>
        /// <param name="cause_code"></param>
        /// <param name="iz"></param>
        public void OnZoiperCallReject(string cause_code, IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = "Call incoming\n";
            });
        }
        /// <summary>
        /// This method appears when someone calls us,
        /// to answer you have to use "iz" variable
        /// </summary>
        /// <param name="iz"></param>
        public void OnZoiperCallIncoming(IZoiperCall iz)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = "Call incoming\n";
            });
        }
        public void OnZoiperCallHoldLocal(IZoiperCall iz)
        {
            /// Fill when you want to do something extra
        }
        public void OnZoiperCallUnholdLocal(IZoiperCall iz)
        {
            /// Fill when you want to do something extra
        }
        /// <summary>
        /// This method appears when change the current user 
        /// </summary>
        /// <param name="contact"></param>
        public void OnZoiperContactChange(IZoiperContact contact)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLog.Text = "Contact changed\n";
            });
        }
        public void OnZoiperContactStatus(IZoiperContact contact, string status)
        {
            /// Fill when you want to do something extra
        }
        public void OnZoiperAccountRegister(IZoiperAccount account)
        {
            /// Fill when you want to do something extra
        }
        public void OnZoiperAccountUnregister(IZoiperAccount account)
        {
            /// Fill when you want to do something extra
        }
        /// <summary>
        /// This method appears if the account registration fail
        /// </summary>
        /// <param name="account"></param>
        /// <param name="cause_code"></param>
        public void OnZoiperAccountRegisterFail(IZoiperAccount account, string cause_code)
        {
            this.Dispatcher.Invoke(() =>
            {
                txtLable.Text = "OnZoiperAccountRegisterFail\n" + cause_code + "\n";
            });
        }
        /// <summary>
        /// The class constructor
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            txtLog.Text = "Hello\n";
            btnHold.IsEnabled = false;
        }
        /// <summary>
        /// Use this method to call on to the selected account
        /// </summary>
        public void fCall()
        {
            txtCaller.Text = "To: " + txtInit.Text;
            calls[numCalls] = myZoiper.Dial(txtInit.Text, 0);
            bTimes[numCalls] = (true);
            txtLog.Text += "Calling " + txtInit.Text + "\n";
            txtInit.Text = "";
            btnHold.IsEnabled = true;
            txtAcound.Text = "From: " + calls[numCalls].Account.Name + " ";
        }
        /// <summary>
        /// Initialise all needed functions and variables 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnInit_Click(object sender, RoutedEventArgs e)
        {

            myZoiper = new ZoiperAPI.ZoiperAPI();

            myZoiper.OnZoiperCallHang += OnZoiperCallHang;
            myZoiper.OnZoiperCallRing += OnZoiperCallRing;
            myZoiper.OnZoiperCallHold += OnZoiperCallHold;
            myZoiper.OnZoiperCallUnhold += OnZoiperCallUnhold;
            myZoiper.OnZoiperCallAccept += OnZoiperCallAccept;
            myZoiper.OnZoiperCallIncoming += OnZoiperCallIncoming;

            myZoiper.OnZoiperContactChange += OnZoiperContactChange;
            myZoiper.OnZoiperAccountRegister += OnZoiperAccountRegister;
            myZoiper.OnZoiperAccountUnregister += OnZoiperAccountUnregister;

            calls.Add(callTemp);
            bTimes.Add(false);

            txtLog.Text += "Init Success\n";
            newTimer = new DispatcherTimer();
            newTimer.Interval = TimeSpan.FromSeconds(1);
            newTimer.Tick += OnTimerTick;

            cbAccounts.Items.Add("Account 1300");
            cbAccounts.Items.Add("Account 1200");

            txtVolumeDisplay.Text = myZoiper.GetConfig().AudioOutputVolume.ToString();
            btnSlider.Value = double.Parse(txtVolumeDisplay.Text);
        }
        /// <summary>
        /// This method appears on every seccond
        /// and get the information from the call
        /// for the duration of the call and the codec
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        public void OnTimerTick(Object sender, EventArgs args)
        {
            if (bTimes[numCalls])
            {
                lastCall = calls[numCalls].Duration;
                txtTime.Text = lastCall + " ";
                txtTime.Text += calls[numCalls].CurrentCodec;
            }
        }
        /// <summary>
        /// It depends of witch account has selected call it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCall_Click(object sender, RoutedEventArgs e)
        {
            if (txtInit.Text != "")
            {
                if (cbAccounts.SelectedItem.ToString() == "Account 1300")
                {
                    myZoiper.UseAccount("1300@10.2.1.9:6060");
                    fCall();
                }
                if (cbAccounts.SelectedItem.ToString() == "Account 1200")
                {
                    myZoiper.UseAccount("1200@10.2.1.9:6060");
                    fCall();
                }
            }
        }
        /// <summary>
        /// Ends the call
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnHangUp_Click(object sender, RoutedEventArgs e)
        {
            calls[numCalls].Hang();
            bTimes[numCalls] = false;
            //txtLable.Text = "Down\n";
            btnHold.IsEnabled = false;
        }
        /// <summary>
        /// Clear the history
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClearLog_Click(object sender, RoutedEventArgs e)
        {
            txtLog.Text = "";
        }
        /// <summary>
        /// Adds 1 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOne_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "1";
        }
        /// <summary>
        /// Adds 2 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTue_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "2";
        }
        /// <summary>
        /// Adds 3 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnThree_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "3";
        }
        /// <summary>
        /// Adds 4 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFour_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "4";
        }
        /// <summary>
        /// Adds 5 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFive_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "5";
        }
        /// <summary>
        /// Adds 6 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSix_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "6";
        }
        /// <summary>
        /// Adds 7 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSeven_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "7";
        }
        /// <summary>
        /// Adds 8 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEight_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "8";
        }
        /// <summary>
        /// Adds 9 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNine_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "9";
        }
        /// <summary>
        /// Adds 0 on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnZero_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "0";
        }
        /// <summary>
        /// Adds * on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAsterisk_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "*";
        }
        /// <summary>
        /// Adds # on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDies_Click(object sender, RoutedEventArgs e)
        {
            txtInit.Text += "#";
        }
        /// <summary>
        /// Clear the last symbol on the display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            string temp = "";
            if (txtInit.Text != "")
            {
                temp = txtInit.Text.Substring(0, txtInit.Text.Length - 1);
                txtInit.Text = temp;
            }
        }
        /// <summary>
        /// Hold/Unhold the current call
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnHold_Click(object sender, RoutedEventArgs e)
        {
            if (bHold)
            {
                btnHold.Content = "UnHold";
                bHold = false;
                calls[numCalls].Hold();
                txtLable.Text = "Hold\n";
            }
            else
            {
                btnHold.Content = "Hold";
                bHold = true;
                calls[numCalls].UnHold();
                txtLable.Text = "Up\n";
            }
        }
        /// <summary>
        /// Set the first call current
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOne1_Click(object sender, RoutedEventArgs e)
        {
            txtAcound.Text = "From: " + calls[0].Account.Name;
            txtCaller.Text = "To: " + calls[0].Phone;
            numCalls--;
            calls[0].UnHold();
        }
        /// <summary>
        /// Set the second call current
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTue1_Click(object sender, RoutedEventArgs e)
        {
            txtAcound.Text = "From: ";
            txtCaller.Text = "To: ";
            numCalls++;
            callTemp = null;
            calls.Add(callTemp);
            bTimes.Add(false);
            calls[0].Hold();
        }
        /// <summary>
        /// Transfer the call to choosen new caller
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTransfer_Click(object sender, RoutedEventArgs e)
        {
            if (txtInit.Text != "")
            {
                calls[numCalls].Transfer(txtInit.Text);

                txtLog.Text += "Calling " + txtInit.Text + "\n";
                txtInit.Text = "";
            }
        }
        /// <summary>
        /// Recive the number of active calls
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNumCalls_Click(object sender, RoutedEventArgs e)
        {
            txtNumCalls.Text = "Number of calls: " + myZoiper.NumberOfCalls;
        }
        /// <summary>
        /// Mute speaker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnVolumeDown_Click(object sender, RoutedEventArgs e)
        {
            //myZoiper.MuteSpeaker = !myZoiper.MuteSpeaker;
            //myZoiper.MuteMicrophone = !myZoiper.MuteMicrophone;
            /*
            if (myZoiper.MuteMicrophone) btnVolumeDown.Content = "Mute";
            else btnVolumeDown.Content = "Mic";
            */
            myZoiper.GetConfig().AudioOutputVolume = 0;
            txtVolumeDisplay.Text = myZoiper.GetConfig().AudioOutputVolume.ToString();
            btnSlider.Value = double.Parse(txtVolumeDisplay.Text);

        }
        /// <summary>
        /// Put the speaker volume to max 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnVolumeUp_Click(object sender, RoutedEventArgs e)
        {
            myZoiper.GetConfig().AudioOutputVolume = 100;
            txtVolumeDisplay.Text = myZoiper.GetConfig().AudioOutputVolume.ToString();
            btnSlider.Value = double.Parse(txtVolumeDisplay.Text);
        }
        /// <summary>
        /// Change the speaker volume manualy
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            txtVolumeDisplay.Text = btnSlider.Value.ToString();
            myZoiper.GetConfig().AudioOutputVolume = int.Parse(txtVolumeDisplay.Text);
        }
    }
}
